@@ -18,24 +18,29 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
     server.assign_model(model())
 
     nodes = [node(i, device) for i in range(num_nodes)]
-    local_models = [model() for i in range(num_nodes)]
-    local_loss = [objective() for i in range(num_nodes)]
+    # local_models = [model() for i in range(num_nodes)]
+    # local_loss = [objective() for i in range(num_nodes)]
 
     for i in range(num_nodes):
         # data
         # print(len(train_splited[i]), len(test_splited[i]))
         nodes[i].assign_train(DataLoader(train_splited[i], batch_size=batch_size, shuffle=True))
-        nodes[i].assign_test(DataLoader(test_splited[i],batch_size=batch_size, shuffle=False))
+        nodes[i].assign_test(DataLoader(test_splited[i], batch_size=batch_size, shuffle=False))
         # model
-        nodes[i].assign_model(local_models[i])
+        nodes[i].assign_model(model())
         # objective
-        nodes[i].assign_objective(local_loss[i])
+        nodes[i].assign_objective(objective())
         # optim
         nodes[i].assign_optim(optimizer(nodes[i].model.parameters()))
+    
+    del train_splited, test_splited
 
     # initialize parameters to nodes
     server.distribute(nodes, list(range(num_nodes)))
 
+    # initialize K cluster model
+    cluster_models = [model() for i in range(K)]
+
     # train!
     for i in range(global_rounds):
         print('-------------------Global round %d start-------------------' % (i))
@@ -51,13 +56,16 @@ def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer,
         for i in range(K):
             server.aggregate(nodes, [j for j in list(range(num_nodes)) if nodes[j].label==i])
             server.distribute(nodes, [j for j in list(range(num_nodes)) if nodes[j].label==i])
+            cluster_models[k].load_state_dict(server.model.state_dict())
         # test accuracy
         for j in range(num_nodes):
             nodes[j].local_test()
         server.acc(nodes, list(range(num_nodes)))
     
     # log
-    if not reg:
+    if reg:
         log(os.path.basename(__file__)[:-3] + '_' + str(K) + '_' + str(reg) + '_' + split_para, nodes, server)
     else:
-        log(os.path.basename(__file__)[:-3] + '_' + str(K) + '_' + split_para, nodes, server)
\ No newline at end of file
+        log(os.path.basename(__file__)[:-3] + '_' + str(K) + '_' + split_para, nodes, server)
+
+    return cluster_models
\ No newline at end of file
